/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                         
** Fichier : kernel_mod_receptivities.c                                     
** Description : Module noyau receptivites                                  
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_receptivities.c
 * \brief Module : kernel, Module noyau receptivites
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_receptivities.h"
/*
TSKERNEL_RES
*/
int TSKERNEL_RES_receptivities_set(int i,int j,int value){//Use TSKERNEL_MOD_receptivities_set(...) to set receptivities
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_receptivities);
	#else
		sem_wait(&TSKERNEL_SEM_receptivities);
	#endif
	TSKERNEL_RECEPTIVITIES[i][j]=value;
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_receptivities);
	#else
		sem_post(&TSKERNEL_SEM_receptivities);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_RES_receptivities_get(int i,int j){//Use TSKERNEL_MOD_receptivities_get(...) to get receptivities
	int value;
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_receptivities);
	#else
		sem_wait(&TSKERNEL_SEM_receptivities);
	#endif
	value=TSKERNEL_RECEPTIVITIES[i][j];
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_receptivities);
	#else
		sem_post(&TSKERNEL_SEM_receptivities);
	#endif
	return value;
}

/*
TSKERNEL_MOD_receptivities
*/
int TSKERNEL_MOD_receptivities_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"receptivities : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_receptivities=rt_sem_init(TSKERNEL_SEM_receptivities,1);
	#else
		sem_init(&TSKERNEL_SEM_receptivities,0,1);
	#endif
	int i=0;
	int j=0;
	for(i=0;i<RECEPTIVITIES_READ_ALL_NB;i++){
		for(j=0;j<RECEPTIVITIES_BITS_NB;j++){
			TSKERNEL_RES_receptivities_set(i,j,RECEPTIVITIES_STATE_INIT);
		}
	}
	TSKERNEL_MOD_receptivities_refreshAll(1);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"receptivities : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_receptivities_unload(void){
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_receptivities);
	#else
		sem_destroy(&TSKERNEL_SEM_receptivities);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"receptivities : unload");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_receptivities_get(int octet,int bit){
	if (octet>=RECEPTIVITIES_READ_ALL_NB || bit>=RECEPTIVITIES_BITS_NB){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"receptivities : invalid receptivity [%d,%d]",octet,bit);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_RECEPTIVITY;
	}
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	if(TSKERNEL_RES_kernel_simul_state_get()==TSKERNEL_SIMUL_OFF) 
		return TSKERNEL_RES_receptivities_get(octet,bit);
	else
		return RECEPTIVITIES_STATE_ON;
}

int TSKERNEL_MOD_receptivities_set(int octet,int bit,int value){
	if (octet>=RECEPTIVITIES_READ_ALL_NB || bit>=RECEPTIVITIES_BITS_NB || (value!=RECEPTIVITIES_STATE_OFF && value!=RECEPTIVITIES_STATE_ON && RECEPTIVITIES_STATE_INIT)){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"receptivities : invalid receptivity [%d,%d]",octet,bit);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_RECEPTIVITY;
	}
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"receptivities : receptivity [%d,%d] changed to %d",octet,bit,value);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,message);
	free(message);
	if (TSKERNEL_RES_kernel_simul_run_get()==TSKERNEL_RUN_ON) TSGUI_update(TSKERNEL_THREADSAFE_OFF);
	return TSKERNEL_RES_receptivities_set(octet,bit,value);
}

int TSKERNEL_MOD_receptivities_refreshAll(int clearOnExit){
	int msg1;
	int i=0;
	int j=0;
	if(clearOnExit){
		msg1 = RECEPTIVITIES_INS_TRANSMISSION_EFFACEMENT;
	} else {
		msg1 = RECEPTIVITIES_INS_TRANSMISSION_MAINTIEN;
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"receptivities : global refresh begin");
	TSKERNEL_CMD_interface_sendTwo(msg1,RECEPTIVITIES_READ_ALL_3_DECODERS);
	for(i=0;i<RECEPTIVITIES_READ_ALL_NB;i++){
		for(j=0;j<RECEPTIVITIES_BITS_NB;j++){
			TSKERNEL_RES_receptivities_set(i,j,TSKERNEL_CMD_interface_read());
		}
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"receptivities : global refresh end");
	return TSKERNEL_NORMAL_EXIT;	
}

int TSKERNEL_MOD_receptivities_isValidDecoder(int decoder_num){
	if(decoder_num<1 || decoder_num>3){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"receptivities : invalid decoder %d",decoder_num);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_DECODER;
	} else return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_receptivities_refresh(int decoder_num,int clearOnExit){
	int msg1;
	int msg2;
	int i=0;
	int j=0;
	if(TSKERNEL_MOD_receptivities_isValidDecoder(decoder_num)!=TSKERNEL_NORMAL_EXIT)
		return TSKERNEL_INVALID_DECODER;
	switch (decoder_num){
		case 1:
			msg2=RECEPTIVITIES_READ_SELECT_DECODER_1;
		break;
		case 2:
			msg2=RECEPTIVITIES_READ_SELECT_DECODER_2;
		break;
		case 3:
			msg2=RECEPTIVITIES_READ_SELECT_DECODER_3;
		break;
	}
	if(clearOnExit){
		msg1 = RECEPTIVITIES_INS_TRANSMISSION_EFFACEMENT;
	} else {
		msg1 = RECEPTIVITIES_INS_TRANSMISSION_MAINTIEN;
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"receptivities : selective refresh begin");
	TSKERNEL_CMD_interface_sendTwo(msg1,msg2);
	for(i=decoder_num-1;i<(decoder_num-1)+RECEPTIVITIES_READ_SELECT_NB;i++){
		for(j=0;j<RECEPTIVITIES_BITS_NB;j++){
			TSKERNEL_RES_receptivities_set(i,j,TSKERNEL_CMD_interface_read());
		}
	}
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_DEBUG,"receptivities : selective refresh end");
	return TSKERNEL_NORMAL_EXIT;
}
