/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                       
** Fichier : kernel_mod_serial.c                                     
** Description : Gestion de la ligne serie                              
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_serial.c
 * \brief Module : kernel, Module noyau gestion de la ligne serie
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_serial.h"

/*
TSKERNEL_MOD_serial
*/
int TSKERNEL_MOD_serial_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"serial : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_serial_lock=rt_sem_init(TSKERNEL_SEM_serial_lock,1);
	#else
		sem_init(&TSKERNEL_SEM_serial_lock,0,1);
	#endif
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_serial_lock);
		TSKERNEL_MOD_serial_line=rt_spopen(SERIAL_DEV,SERIAL_BAUD,SERIAL_NUMBITS,SERIAL_STOPBITS,RT_SP_PARITY_NONE,RT_SP_HW_FLOW,RT_SP_FIFO_DISABLE);
		if(TSKERNEL_MOD_serial_line) {
		      switch(TSKERNEL_MOD_serial_line) {
			 case -ENODEV:
				TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"serial : unable to open port, no device");
			 break;
			 case -EINVAL:
				TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"serial : unable to open port, invalid value");
			 break;
			 case -EADDRINUSE:
				TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"serial : unable to open port, address in use");
			 break;
			 default:
				TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"serial : unable to open port, unknown error");
			 break;
		      }
		}
		rt_spclear_rx(SERIAL_DEV);
		rt_spclear_tx(SERIAL_DEV);
		rt_sem_signal(TSKERNEL_SEM_serial_lock);
	#else
		sem_wait(&TSKERNEL_SEM_serial_lock);
		struct termios ligne_serie;
		speed_t vitesse;

		//Ouverture port
		TSKERNEL_MOD_serial_line = open(SERIAL_DEV, O_RDWR | O_NONBLOCK);

		//Lecture configuration
		tcgetattr(TSKERNEL_MOD_serial_line, &ligne_serie);
		//Nouvelle configuration
		ligne_serie.c_cflag |= (CS8); //Code a 8 bits
		ligne_serie.c_cflag |= (CSTOPB); //2 bits Stop
		ligne_serie.c_cflag &= ~(PARENB); //Pas de parite
		vitesse = SERIAL_BAUD;
		cfsetispeed(&ligne_serie, vitesse); //Vitesse en reception
		cfsetospeed(&ligne_serie, vitesse); //Vitesse en emission

		//Envoi nouvelle configuration
		tcsetattr(TSKERNEL_MOD_serial_line, TCSANOW, &ligne_serie);
		sem_post(&TSKERNEL_MOD_serial_line);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"serial : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_serial_unload(void){
	//Fermeture port
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_serial_lock);
		rt_spclose(SERIAL_DEV);
		rt_sem_signal(TSKERNEL_SEM_serial_lock);
	#else
		sem_wait(&TSKERNEL_SEM_serial_lock);
		close(TSKERNEL_MOD_serial_line);
		sem_post(&TSKERNEL_MOD_serial_line);
	#endif
	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_serial_lock);
	#else
		sem_destroy(&TSKERNEL_SEM_serial_lock);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"serial : unload");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_serial_send(int lg,char* str){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_serial_lock);
		if(rt_spwrite(SERIAL_DEV,str,lg)<0){
			TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"serial : unable to write to device");
			return TSKERNEL_INVALID_SERIAL_WRITE;
		}
		rt_sem_signal(TSKERNEL_SEM_serial_lock);
	#else
		sem_wait(&TSKERNEL_SEM_serial_lock);
		while (lg > 0) {
			write(TSKERNEL_MOD_serial_line,str, 1);
			str++;
			lg = lg - 1;
		}
		sem_post(&TSKERNEL_MOD_serial_line);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_serial_read(void){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_serial_lock);
		unsigned char buf[SERIAL_BUFLEN_READ] = {0};
   		if(rt_spread(SERIAL_DEV, &buf[0], SERIAL_BUFLEN_READ)){
			rt_sem_signal(TSKERNEL_SEM_serial_lock);
      			return buf[0];
		}
		rt_sem_signal(TSKERNEL_SEM_serial_lock);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"serial : unable to read from device");
		return TSKERNEL_INVALID_SERIAL_READ;
	#else
		sem_wait(&TSKERNEL_SEM_serial_lock);
		int byteRead;
		char char_in;
		while ((byteRead = read(TSKERNEL_MOD_serial_line, &char_in, 1)) == -1);
		//	byteRead = read (TSKERNEL_MOD_serial_line,&char_in,1);
		//	if (byteRead == -1) char_in = -1;
		sem_post(&TSKERNEL_MOD_serial_line);
		return char_in;
	#endif
}

int TSKERNEL_MOD_serial_cleanBuffer(void){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_serial_lock);
		rt_spclear_rx(SERIAL_DEV);
		rt_spclear_tx(SERIAL_DEV);
		rt_sem_signal(TSKERNEL_SEM_serial_lock);
	#else
		sem_wait(&TSKERNEL_SEM_serial_lock);
		int byteRead;
		char c;
		byteRead = 0;
		while ((byteRead = read(TSKERNEL_MOD_serial_line, &c, 1)) != -1);
		{
			//printf(" Purge du caractère %3d \n", c);
		}
		sem_post(&TSKERNEL_MOD_serial_line);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}


