/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : kernel                                                       
** Fichier : kernel_mod_socket.c                                     
** Description : Gestion des sockets                           
------------------------------------------------------------------------------*/
/**
 * \file kernel_mod_socket.c
 * \brief Module : kernel, Module noyau socket
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include "kernel_mod_socket.h"

/*
TSKERNEL_MOD_socket
*/
int TSKERNEL_MOD_socket_load(void){
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"socket : load begin");
	//Initialisation des semaphores
	#ifdef TS_RTAI
		TSKERNEL_SEM_socket_lock=rt_sem_init(TSKERNEL_SEM_socket_lock,1);
	#else
		sem_init(&TSKERNEL_SEM_socket_lock,0,1);
	#endif

	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_socket_lock);
	#else
		sem_wait(&TSKERNEL_SEM_socket_lock);
	#endif
	//Init RECV socket
	TSKERNEL_MOD_socket_recv_socket_id = TSKERNEL_MOD_socket_createTcpClient(SOCKET_HOST,SOCKET_PORT_RECV);
	if (-1 == TSKERNEL_MOD_socket_recv_socket_id) return TSKERNEL_INVALID_SOCKET;

	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_socket_lock);
	#else
		sem_post(&TSKERNEL_SEM_socket_lock);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"socket : load end");
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_socket_unload(void){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_socket_lock);
	#else
		sem_wait(&TSKERNEL_SEM_socket_lock);
	#endif

	//Close sockets
	shutdown(TSKERNEL_MOD_socket_recv_socket_id, 2);
	close(TSKERNEL_MOD_socket_recv_socket_id);

	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_socket_lock);
	#else
		sem_post(&TSKERNEL_SEM_socket_lock);
	#endif

	//Destruction des semaphores
	#ifdef TS_RTAI
		rt_sem_delete(TSKERNEL_SEM_socket_lock);
	#else
		sem_destroy(&TSKERNEL_SEM_socket_lock);
	#endif
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INFO,"socket : unload");
	return	TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_socket_createTcpClient(const char *hostname, int port){
	struct hostent *host_address;
	struct sockaddr_in sockname;
	int optval;
	int socket_id;

	// Obtention de l'adresse de la machine distante
	if (NULL == (host_address = gethostbyname (hostname))){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"socket : unable to find host %s", hostname);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_SOCKET;
	}

	// Création d'une socket
	if (-1 == (socket_id = socket (PF_INET, SOCK_STREAM, 0))){
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,"socket : unable to create socket");
		return TSKERNEL_INVALID_SOCKET;
	}

	/* Changement d'un paramètre de la socket pour permettre une réutilisation
 	 * immédiate après sa fermeture
 	 */
	optval = 1;
	setsockopt (socket_id, SOL_SOCKET, SO_REUSEADDR, &optval, sizeof (int));

	// Connexion de la socket au serveur distant
	sockname.sin_family = host_address->h_addrtype;
	sockname.sin_port = htons (port);
	memcpy ((char *) &(sockname.sin_addr.s_addr), host_address->h_addr,host_address->h_length);
	if (-1 ==(connect(socket_id, (struct sockaddr *) &sockname,sizeof (struct sockaddr_in)))){
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"socket : unable to connect to host %s", hostname);
		TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_ERROR,message);
		free(message);
		return TSKERNEL_INVALID_SOCKET;
	}
	return (socket_id);
}

int TSKERNEL_MOD_socket_read(char* recv_c){
	int ret=0;

	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_socket_lock);
	#else
		sem_wait(&TSKERNEL_SEM_socket_lock);
	#endif
	ret=read(TSKERNEL_MOD_socket_recv_socket_id,recv_c, sizeof(char));
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_socket_lock);
	#else
		sem_post(&TSKERNEL_SEM_socket_lock);
	#endif
	return ret;
}

int TSKERNEL_MOD_socket_send_1int(int op_code,int param1){
	#ifdef TS_RTAI
		rt_sem_wait(TSKERNEL_SEM_socket_lock);
	#else
		sem_wait(&TSKERNEL_SEM_socket_lock);
	#endif
	char* send_str=(char*) malloc(255*sizeof(char));
	sprintf(send_str,"%c%c%d%c%d%c%c",API_QUERY_START_CHAR,API_DELIMITER_CHAR,op_code,API_DELIMITER_CHAR,param1,API_DELIMITER_CHAR,API_QUERY_END_CHAR);
	write (TSKERNEL_MOD_socket_recv_socket_id,send_str,strlen(send_str));
	char* message=(char*)malloc(1024*sizeof(char));
	sprintf(message,"socket : send to socket : %s", send_str);
	TSKERNEL_trace(TSKERNEL_CODE_USER,TSKERNEL_TRACE_INTERNAL,message);
	free(message);
	free(send_str);
	#ifdef TS_RTAI
		rt_sem_signal(TSKERNEL_SEM_socket_lock);
	#else
		sem_post(&TSKERNEL_SEM_socket_lock);
	#endif
	return TSKERNEL_NORMAL_EXIT;
}

int TSKERNEL_MOD_socket_sendAnswer(int param1){
	return TSKERNEL_MOD_socket_send_1int(API_OPCODE_ANSWER,param1);
}
