/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : main                                                       
** Fichier : trainsim.c                                     
** Description : Lancement noyau + application                              
------------------------------------------------------------------------------*/
/**
 * \file trainsim.c
 * \brief Module : main
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#ifndef TS_RTAI
	#include <pthread.h>
#endif
#include "tasks.h"
#include "kernel.h"
#include "const_global.h"
#include "const_kernel.h"
#include "const_tasks.h"
#include "gui_interface.h"

#ifdef TS_RTAI
static RT_TASK *masterTask;
#endif

static int TS_trace_level=TS_TRACE_DEBUG;

int TS_trace(int type,char* message){
	if (type<=TS_trace_level){
		switch(type){
			case TS_TRACE_ERROR:
				printf("[TSMAIN_ERROR  ] %s\n",message);
			break;
			case TS_TRACE_INFO:
				printf("[TSMAIN_INFO   ] %s\n",message);
			break;
			case TS_TRACE_DEBUG:
				printf("[TSMAIN_DEBUG  ] %s\n",message);
			break;
			case TS_TRACE_INTERNAL:
				printf("[TSMAIN_INTER  ] %s\n",message);
			break;
		}
	}
	return TS_NORMAL_EXIT;
}


void TS_SIG_handler_int(int dummy){
	//Stop kernel
	TSKERNEL_stop();
	TSKERNEL_end();
	#ifdef TS_RTAI
		//Stop RTAI timer
		stop_rt_timer();
		TS_trace(TS_TRACE_DEBUG,"main : stop RTAI timer");
		//Delete master task
		rt_task_delete(masterTask);
		TS_trace(TS_TRACE_DEBUG,"main : master task delete");
	#endif
	TS_trace(TS_TRACE_DEBUG,"main : end");
	signal(SIGINT,SIG_DFL);
	exit(TS_ERROR_EXIT);
}

int main(int argc, char** argv){
	#ifdef TS_RTAI 
		unsigned long masterTask_name = nam2num("TS_MASTER");
	#endif
	//Assign trace level
	TS_trace_level=TS_TRACE_NO;
	TS_trace(TS_TRACE_DEBUG,"main : begin");
	//Signal connect
	signal(SIGINT,TS_SIG_handler_int);

	#ifdef TS_RTAI
		//Init master task (make main() a RTAI task)
	 	if (!(masterTask = rt_task_init(masterTask_name,1,0,0))) {
			TS_trace(TS_TRACE_ERROR,"main : cannot init master task"); 
			exit(TS_ERROR_EXIT);
		}
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"main : init master task with address = %p",masterTask);
		free(message);
		TS_trace(TS_TRACE_DEBUG,message);

		//Scheduler mode
	//	rt_set_oneshot_mode();
		rt_set_periodic_mode();
	
		//Start RTAI timer
		start_rt_timer(0);
		TS_trace(TS_TRACE_DEBUG,"main : start RTAI timer");
	#endif

	//GUI Init
	TS_trace(TS_TRACE_DEBUG,"main : init gui thread");
	//Make GTK thread safe	
	g_thread_init(NULL);
	gdk_threads_init();

	#ifdef ENABLE_NLS
		bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
		bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
		textdomain (GETTEXT_PACKAGE);
	#endif

	gtk_set_locale ();
	gtk_init(&argc, &argv);

	add_pixmap_directory ("./pixmaps");
	TSGUI_mainWindow = TSGUI_create_mainWindow ();
	
	//Kernel thread create
	TS_trace(TS_TRACE_DEBUG,"main : init kernel thread");
	TSKERNEL_CMD_trace_setLevel(TSKERNEL_TRACE_DEBUG);
	TS_tasks_indx[TS_TASKS_KERNEL]=TS_TASKS_KERNEL;
	TS_tasks_params_s kernelParams;
	kernelParams.mytask_indx=TS_tasks_indx[TS_TASKS_KERNEL];
	kernelParams.rt_state=TS_TASKS_SOFT_RT;
	kernelParams.taskName="kernel";
	kernelParams.taskBody=TS_tasks_kernel_body;
	kernelParams.taskEnd=NULL;//TS_tasks_kernel_end;
	#ifdef TS_RTAI
		if (!(TS_tasks[TS_TASKS_KERNEL] = rt_thread_create(TS_tasks_generic, &kernelParams,10000))) {
	#else
		if (pthread_create(&TS_tasks[TS_TASKS_KERNEL],0,TS_tasks_generic, &kernelParams)!=0) {
	#endif
		TS_trace(TS_TRACE_ERROR,"main : cannot create kernel thread"); 
		exit(TS_ERROR_EXIT);
	}

	//GUI thread create
	TS_trace(TS_TRACE_DEBUG,"main : init gui thread");
	TSKERNEL_CMD_trace_setLevel(TSKERNEL_TRACE_DEBUG);
	TS_tasks_indx[TS_TASKS_GUI]=TS_TASKS_GUI;
	TS_tasks_params_s guiParams;
	guiParams.mytask_indx=TS_tasks_indx[TS_TASKS_GUI];
	guiParams.rt_state=TS_TASKS_SOFT_RT;
	guiParams.taskName="gui";
	guiParams.taskBody=TS_tasks_gui_body;
	guiParams.taskEnd=NULL;
	#ifdef TS_RTAI
		if (!(TS_tasks[TS_TASKS_GUI] = rt_thread_create(TS_tasks_generic, &guiParams,10000))) {
	#else
		if (pthread_create(&TS_tasks[TS_TASKS_GUI],0,TS_tasks_generic, &guiParams)!=0) {
	#endif
		TS_trace(TS_TRACE_ERROR,"main : cannot create gui thread"); 
		exit(TS_ERROR_EXIT);
	}

	#ifdef TS_NOGUI
	int v=0;	while(1){v--;v++;}
	#endif

	//Wait for all threads to finish
	int i=0;
	for (i=0;i<TS_TASKS_NB_MAIN;i++) {
		#ifdef TS_RTAI
			rt_thread_join(TS_tasks[i]);
		#else
			pthread_join(TS_tasks[i],NULL);
		#endif
	}
	TS_trace(TS_TRACE_DEBUG,"main : all threads end");

	//Stop kernel
	TSKERNEL_stop();
	TSKERNEL_end();
	#ifdef TS_RTAI
		//Stop RTAI timer
		stop_rt_timer();
		TS_trace(TS_TRACE_DEBUG,"main : stop RTAI timer");

		//Delete master task
		rt_task_delete(masterTask);
		TS_trace(TS_TRACE_DEBUG,"main : master task delete");
	#endif
	TS_trace(TS_TRACE_DEBUG,"main : end");
	return TS_NORMAL_EXIT;
}
