/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : api                                                        
** Fichier : api_usr.h                                    
** Description : API to use kernel for endusers, all API calls will be 
**               translated to kernel calls
------------------------------------------------------------------------------*/
/**
 * \file api_usr.h
 * \brief Module : api, API to use kernel for endusers, all API calls will be 
 *               translated to kernel calls
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#ifndef API_USR_H
#define API_USR_H
#include "usr_const_global.h"
#include "usr_tasks.h"
#include "usr_const_tasks.h"
#include "usr_socket.h"
#include "const_api.h"
#include "const_trains.h"
#include "usr.h"

/*
TSAPI
*/
int TSAPI_init(void);
int TSAPI_end(void);
int TSAPI_trace(int type,char* message);

/*
TSAPI_aiguillages
*/
/**
 * \fn int TSAPI_aiguillages_setPosition(char* aiguillage_num,char* aiguillage_pos)
 * \brief Change la position d'un aiguillage
 * \param aiguillage_num nom de l'aiguillage
 * \param aiguillage_pos nouvelle position
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_aiguillages_setPosition(char* aiguillage_num,char* aiguillage_pos);

/*
TSAPI_blocks
*/
/**
 * \fn int TSAPI_blocks_mutex_P(int block_num,int train_num)
 * \brief Prend le mutex du block
 * \param block_num numero du block
 * \param train_num numero du trains
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_blocks_mutex_P(int block_num,int train_num);
/**
 * \fn int TSAPI_blocks_mutex_V(int block_num,int train_num)
 * \brief Libere le mutex du block
 * \param block_num numero du block
 * \param train_num numero du trains
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_blocks_mutex_V(int block_num,int train_num);

/*
TSAPI_circuit
*/
/**
 * \fn int TSAPI_circuit_relachement(void)
 * \brief Envoie l'ordre de relachement
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_circuit_relachement(void);
/**
 * \fn int TSAPI_circuit_disjonction(void)
 * \brief Envoie l'ordre de disjonction
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_circuit_disjonction(void);
/**
 * \fn int TSAPI_circuit_conjonction(void)
 * \brief Envoie l'ordre de conjonction
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_circuit_conjonction(void);

/*
TASPI_detelages
*/
/**
 * \fn int TSAPI_detelages_getPosition(int detelage_num)
 * \brief Renvoie la position d'un rail de detelage
 * \param detelage_num numéro du rail de detelage
 * \return int code position (see const_detelages.h)
 */
int TSAPI_detelages_getPosition(int detelage_num);
/**
 * \fn int TSAPI_detelages_setPosition(char* detelage_name,int pos)
 * \brief Change la position d'un rail de detelage
 * \param detelage_name nom du rail de detelage
 * \param pos nouvelle position
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_detelages_setPosition(char* detelage_name,int pos);

/*
TASPI_receptivities
*/
/**
 * \fn int TSAPI_receptivities_get(int octet,int bit)
 * \brief Renvoie l'etat de la receptivite demandee
 * \param octet numero d'octet
 * \param bit numero de bit
 * \return int code etat (see const_receptivities.h)
 */
int TSAPI_receptivities_get(int octet,int bit);

/*
TSAPI_signalisation
*/
/**
 * \fn int TSAPI_signalisation_setState(int signal_num, int state)
 * \brief Change l'etat d'un signal
 * \param signal_num numero de signal
 * \param state etat
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_signalisation_setState(int signal_num, int state);
/**
 * \fn int TSAPI_signalisation_getState(int signal_num)
 * \brief Renvoie l'etat d'un signal
 * \param signal_num numero de signal
 * \return int code etat (see const_signalisation.h)
 */
int TSAPI_signalisation_getState(int signal_num);

/*
TSAPI_trains
*/
/**
 * \fn int TSAPI_trains_getSpeed(int train_num)
 * \brief Renvoie la vitesse d'un train
 * \param train_num numero de train
 * \return int code etat (see const_trains.h)
 */
int TSAPI_trains_getSpeed(int train_num);
/**
 * \fn int TSAPI_trains_getPosition(int train_num)
 * \brief Renvoie la position d'un train
 * \param train_num numero de train
 * \return int code position (see const_trains.h)
 */
int TSAPI_trains_getPosition(int train_num);
/**
 * \fn int TSAPI_trains_setSpeed(int train_num,int speed,int fct)
 * \brief Change la vitesse d'un train
 * \param train_num numero de train
 * \param speed nouvelle vitesse
 * \param fct activation de la fonction
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_trains_setSpeed(int train_num,int speed,int fct);
/**
 * \fn int TSAPI_trains_reverse(int train_num,int fct)
 * \brief Change le sens d'un train
 * \param train_num numero de train
 * \param fct activation de la fonction
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_trains_reverse(int train_num,int fct);
/**
 * \fn int TSAPI_trains_createItinerary(int train_num,int startBlock_num)
 * \brief Creation d'un parcours de train
 * \param train_num numero de train
 * \param startBlock_num numero du block de depart
 * \param cyclic_state état cyclique
 * \param initItinerary fonction d'initialisation du parcours
 * \param itinerary fonction corps du parcours
 * \param endItinerary fonction de fin du parcours
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_trains_createItinerary(int train_num,int startBlock_num,int cyclic_state,void (*initItinerary)(void),void (*itinerary)(void),void (*endItinerary)(void));
/**
 * \fn int TSAPI_trains_deleteItinerary(int train_num)
 * \brief Suppression d'un parcours de train
 * \param train_num numero de train
 * \return int code de contrôle (see const_kernel.h)
 */
int TSAPI_trains_deleteItinerary(int train_num);
#endif
