/*------------------------------------------------------------------------------
** Projet : trainsim                                                         
** Auteurs : Vincent BONESTEVE, Dominique PADIOU                           
** Module : main                                                       
** Fichier : trainsim_USR.c                                     
** Description : Lancement noyau + application                              
------------------------------------------------------------------------------*/
/**
 * \file trainsim_USR.c
 * \brief Module : main, programme utilisateur
 * \author Vincent BONESTEVE, Dominique PADIOU 
 * \version 1.0-stable
 * \date 01/01/2010
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#ifndef TSUSR_RTAI
	#include <pthread.h>
#endif
#include "const_trains.h"
#include "usr_tasks.h"
#include "usr_const_global.h"
#include "usr_const_tasks.h"
#include "usr_const_socket.h"
#include "usr_socket.h"
#include "usr_blocks.h"
#include "api_usr.h"
#include "usr.h"


#ifdef TSUSR_RTAI
static RT_TASK *masterTask;
#endif

int TSUSR_trace(int type,char* message){
	if (type<=TSUSR_trace_level){
		switch(type){
			case TSUSR_TRACE_ERROR:
				printf("[TSUSR_ERROR  ] %s\n",message);
			break;
			case TSUSR_TRACE_INFO:
				printf("[TSUSR_INFO   ] %s\n",message);
			break;
			case TSUSR_TRACE_DEBUG:
				printf("[TSUSR_DEBUG  ] %s\n",message);
			break;
			case TSUSR_TRACE_INTERNAL:
				printf("[TSUSR_INTERN ] %s\n",message);
			break;
		}
	}
	return TSUSR_NORMAL_EXIT;
}


void TSUSR_SIG_handler_int(int dummy){
	//Stop API
	TSAPI_end();

	//Close socket
	TSUSR_socket_unload();

	#ifdef TSUSR_RTAI
		//Stop RTAI timer
		stop_rt_timer();
		TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : stop RTAI timer");
		//Delete master task
		rt_task_delete(masterTask);
		TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : master task delete");
	#endif
	TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : end");
	signal(SIGINT,SIG_DFL);
	exit(TSUSR_ERROR_EXIT);
}

int main(int argc, char** argv){
	#ifdef TSUSR_RTAI 
		unsigned long masterTask_name = nam2num("TSUSR_MASTER");
	#endif
	TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : begin");
	//Signal connect
	signal(SIGINT,TSUSR_SIG_handler_int);

	#ifdef TSUSR_RTAI
		//Init master task (make main() a RTAI task)
	 	if (!(masterTask = rt_task_init(masterTask_name,1,0,0))) {
			TSUSR_trace(TSUSR_TRACE_ERROR,"usr : cannot init master task"); 
			//exit(TSUSR_ERROR_EXIT);
		}
		char* message=(char*)malloc(1024*sizeof(char));
		sprintf(message,"usr : init master task with address = %p",masterTask);
		TSUSR_trace(TSUSR_TRACE_DEBUG,message);
		free(message);

		//RTAI Scheduler mode
	//	rt_set_oneshot_mode();
		rt_set_periodic_mode();
	
		//Start RTAI timer
		start_rt_timer(0);
		TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : start RTAI timer");
	#endif

	//Open socket
	TSUSR_socket_load();

	//Tasks scheduler init
	TSUSR_tasks_sched_init();
	
	//Init blocks semaphores
	TSUSR_blocks_load();

	//Init API
	TSAPI_init();

	//Main launch
	TSUSR_main();

	//Init sync thread (used to sync and join all trains threads (replaced at exec by first trains))
	TSUSR_tasks_params_s sync_params;
	sync_params.mytask_indx=TSUSR_tasks_indx[0];
	sync_params.rt_state=TSUSR_TASKS_SOFT_RT;
	sync_params.cyclic_state=TSUSR_TASKS_CYCLIC;
	char* train_name=(char*) malloc(255*sizeof(char));
	sprintf(train_name,"train%d",TRAINS_NUMS[0]);
	sync_params.taskName=train_name;
	sync_params.taskInit=NULL;
	sync_params.taskBody=NULL;
	sync_params.taskEnd=NULL;
	#ifdef TSUSR_RTAI
		if (!(TSUSR_tasks[0] = rt_thread_create(TSUSR_tasks_generic, &sync_params,10000))) {
	#else
		if (pthread_create(&TSUSR_tasks[0],0,TSUSR_tasks_generic, &sync_params)!=0) {
	#endif
		TSUSR_trace(TSUSR_TRACE_ERROR,"usr : cannot create some train thread"); 
		exit(TSUSR_ERROR_EXIT);
	}

	//Wait for all threads to finish
	int k=0;
	for (k=0;k<TRAINS_NB_LOCOMOTIVES;k++) {
		#ifdef TSUSR_RTAI
			rt_thread_join(TSUSR_tasks[k]);
		#else
			pthread_join(TSUSR_tasks[k],NULL);
		#endif
	}
	TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : all threads end");

	//Stop API
	TSAPI_end();

	//Close socket
	TSUSR_socket_unload();

	//End blocks semaphores
	TSUSR_blocks_unload();

	#ifdef TSUSR_RTAI
		//Stop RTAI timer
		stop_rt_timer();
		TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : stop RTAI timer");

		//Delete master task
		rt_task_delete(masterTask);
		TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : master task delete");
	#endif

	//End scheduler
	TSUSR_tasks_sched_end();

	TSUSR_trace(TSUSR_TRACE_DEBUG,"usr : end");
	return TSUSR_NORMAL_EXIT;
}
